#define NETGEN_COPYRIGHT "Copyright 1990,1991,2002,2004,2005"
#define NETGEN_AUTHOR "Massimo A. Sivilotti, Caltech"
#define NETGEN_DEVELOPER "R. Timothy Edwards, MultiGiG, Inc."

#ifndef NULL
#define NULL 0
#endif

/* various Composition directions */
#define NONE 0
#define HORIZONTAL 1
#define VERTICAL 2

/* Define the following to automatically promote all "nodes" in EXT to
 * "ports" in NETGEN's internal format
 * If we don't do this, hierarchical .ext files cannot be flattened.
 */
#define ALL_EXT_NODES_ARE_PORTS

/* netgen.c */
extern void ReopenCellDef(char *name);
extern void CellDef(char *name);
extern void EndCell(void);
extern void Primitive(void);
extern void Port(char *name);
extern void Node(char *name);
extern void Global(char *name);
extern void UniqueGlobal(char *name);
extern void Instance(char *classname, char *instancename);
extern void PortList(char *prefix, char *list_template);
extern void Cell(char *inststr, char *classname, ...);
extern void Wire(char *instancename, ...);
extern void P(char *inststr, char *gate, char *drain, char *source);
extern void N(char *inststr, char *gate, char *drain, char *source);
extern void B(char *inststr, char *, char *, char *);
extern void E(char *inststr, char *, char *, char *);
extern void Cap(char *inststr, char *, char *, char *);
extern void Res(char *inststr, char *, char *, char *);
extern void join(char *node1, char *node2);
extern void Connect(char *tplt1, char *tplt2);
extern void Place(char *name);
extern void Array(char *Cell, int num);
extern void ActelLib(void);
extern void Flatten(char *name);
extern void FlattenInstancesOf(char *classname);
extern void ConnectAllNodes(char *classname);
extern int  NoDisconnectedNodes;

/* objlist.c */
extern void CellDelete(char *name);
extern void CellRename(char *from, char *to);
extern void CellCopy(char *from, char *to);

extern int Debug;
extern int VerboseOutput; /* set this to 1 to enable extra output */
extern int IgnoreRC;	  /* set this to 1 to ignore capacitance and resistance */
extern int NoOutput;      /* set this to 1 to disable stdout output */
extern int Composition;	  /* direction of composition */
extern int UnixWildcards; /* TRUE if *,?,{},[] only; false if full REGEXP */
/* magic internal flag to restrict searches to recently placed cells */
extern int QuickSearch;
/* does re"CellDef"ing a cell add to it or overwrite it??? */
extern int AddToExistingDefinition;
/* procedure to facilitate generating file/cell/pin names */
char *Str(char *format, ...);


/* these are defined in data.h */
extern void Initialize(void);
extern void InitializeCommandLine(int argc, char **argv);
#ifdef TCL_NETGEN
extern void PrintAllElements(char *cell);
#else
extern void PrintElement(char *cell, char *list_template);
#endif
extern void Fanout(char *cell, char *node, int filter);
extern void PrintCell(char *name);
extern void Query(void);

/* this is defined in xnetgen.h */
void X_main_loop(int argc, char *argv[]); 

/* output file formats; also defined in netfile.h */
extern void Ntk(char *name, char *filename);
extern void Actel(char *name, char *filename);
extern void Wombat(char *name, char *filename);
extern void Ext(char *name);
extern void Sim(char *name);
extern void SpiceCell(char *name, char *filename);
extern void EsacapCell(char *name, char *filename);
extern void WriteNetgenFile(char *name, char *filename);
extern void Ccode(char *name, char *filename);

/* input file formats, these routines return the name of the top-level cell */
extern char *ReadNtk (char *fname);
extern char *ReadExtHier(char *fname);
extern char *ReadExtFlat(char *fname);
extern char *ReadSim(char *fname);
extern char *ReadSpice(char *fname);
extern char *ReadNetgenFile (char *fname);

extern char *ReadNetlist(char *fname);


/* these are defined in place.h */
extern void Embed(char *cellname);
extern void PROTOCHIP(void);

/* these are defined in netcomp.c */
extern void NETCOMP(void);
extern int Compare(char *cell1, char *cell2);
extern int EquivalenceTransistors;  /* set this to 0 if source != drain */

/* this is defined in tclnetgen.c */
#ifdef TCL_NETGEN
extern char *tcl_calloc(size_t, size_t);
#endif

