/*  objlist.c -- core allocation, list generation by regexps */

#define SEPARATOR "/"
#define INSTANCE_DELIMITER "#"
#define PORT_DELIMITER "."
#define PHYSICALPIN "("
#define ENDPHYSICALPIN ")"

#define PORT (-1)
#define GLOBAL (-2)
#define UNIQUEGLOBAL (-3)
#define ALLELEMENTS (-4)	/* for doing searches; e.g., Fanout() */
#define ALLOBJECTS (-5)		/* for doing searches; e.g., Fanout() */
#define NODE 0
#define FIRSTPIN 1
#define IsPort(a) ((a == PORT) || (a == GLOBAL) || (a == UNIQUEGLOBAL))

/* list of objects within a cell */
struct objlist {
  char *name;			/* unique name for the port/node/pin */
  int type;			/* -1 for port,  0 for internal node, 
				   else index of the pin on element */
  char *classname;	/* name of element class; nullstr for ports/nodes */
  char *instance;	/* unique name for the instance */
  int node;		/* the electrical node number of the port/node/pin */
  struct objlist *next;
};

extern struct objlist *LastPlaced; 

#if defined(IBMPC)  || defined(HASH_OBJECTS)
/* disable these for memory-efficiency reasons on PC, unless -DHASH_OBJECTS */

/* #define HASH_OBJECTS */
/* #define OBJHASHSIZE 101 */
/* #define CACHE_NODENAMES */
#else /* not IBMPC */

/* turn this on to speed up object searches */
#define HASH_OBJECTS
#define OBJHASHSIZE 997 /* the size of the object and instance hash lists */
                        /* prime numbers are good choices as hash sizes */
                        /* 101 is a good number for IBMPC */

/* turn this on to cache NodeName targets into array */
#define CACHE_NODENAMES
#endif /* not IBMPC */

/* cell definition for hash table */
struct nlist {
  char *name;
  int dumped;
  int primitive;  /* set TRUE if cell is a primitive */
  struct objlist *cell;
#ifdef HASH_OBJECTS
  struct hashlist **objtab;  /* hash table of object names */
  struct hashlist **insttab; /* hash table of instance names */
#endif
#ifdef CACHE_NODENAMES
  struct objlist **nodename_cache;
  long nodename_cache_maxnodenum;  /* largest node number in cache */
#endif
  void *embedding;   /* this will be cast to the appropriate data structure */
  struct nlist *next;
  int number;
};

extern struct nlist *CurrentCell;
extern struct objlist *CurrentTail;
extern void AddToCurrentCell(struct objlist *ob);
extern void AddInstanceToCurrentCell(struct objlist *ob);
extern void FreeObject(struct objlist *ob);

extern int NumberOfPorts(char *cellname);
extern struct objlist *InstanceNumber(struct nlist *tp, int inst);

extern struct objlist *List(char *list_template);
extern struct objlist *ListExact(char *list_template);
extern struct objlist *ListCat(struct objlist *ls1, struct objlist *ls2);
extern int ListLen(struct objlist *head);
extern int ListLength(char *list_template);
extern struct objlist *LookupObject(char *name, struct nlist *WhichCell);
extern struct objlist *LookupInstance(char *name, struct nlist *WhichCell);
extern struct objlist *CopyObjList(struct objlist *oldlist);
extern void UpdateNodeNumbers(struct objlist *lst, int from, int to);

/* Function pointer to List or ListExact, allowing regular expressions	*/
/* to be enabled/disabled.						*/

extern struct objlist * (*ListPtr)();

extern void PrintCellHashTable(int full);
extern struct nlist *LookupCell(char *s);
extern struct nlist *InstallInCellHashTable(char *name);
extern void InitCellHashTable(void);
extern void ClearDumpedList(void);
#ifdef CPLUSPLUS
extern int (*MagicFunctionPointer)(struct hashlist *np);
#define RecurseCellHashTable(a) ({ MagicFunctionPointer = a;   \
				 MagicRecurseCellHashTable();  \
				 })
extern int MagicRecurseCellHashTable(void);
#else
extern int RecurseCellHashTable(int (*foo)(struct hashlist *np));
#endif
extern struct nlist *FirstCell(void);
extern struct nlist *NextCell(void);

extern char *NodeName(struct nlist *tp, int node);
extern char *NodeAlias(struct nlist *tp, struct objlist *ob);
extern void FreeNodeNames(struct nlist *tp);
extern void CacheNodeNames(struct nlist *tp);


/* enable the following line to debug the core allocator */
/* #define DEBUG_GARBAGE */
   
#ifdef DEBUG_GARBAGE
extern struct objlist *GetObject(void);
extern void FreeString(char *foo);
extern char *strsave(char *s);
extern int match(char *st1, char *st2);
#else /* not DEBUG_GARBAGE */
#define GetObject() ((struct objlist*)calloc(1,sizeof(struct objlist)))
#define FreeString(a) (free(a))
#define strsave(a) strdup(a)

/* generate inline function call for string matching, if we have GCC */
#if defined(__GNUC__) && defined(FAST_STRINGMATCH)
this code deoes not work.  So this is a syntax error.
inline static int stringmatch(char *a, char *b)
{
  while (*a && *b)
    if (*a++ != *b++) return(0);
  return(1);
}
#define match(a,b) (stringmatch((a),(b)))
#else  /* not __GNUC__ && FAST_STRINGMATCH */
#define match(a,b) (!strcmp((a),(b)))
#endif /* not __GNUC__ && FAST_STRINGMATCH */

#endif /* not DEBUG_GARBAGE */

extern int  matchnocase(char *, char *);
extern void GarbageCollect(void);
extern void InitGarbageCollection(void);
extern void AddToGarbageList(struct objlist *head);

extern void TogglePrimitive(char *cellname);

#ifdef HAVE_MALLINFO
void PrintMemoryStats(void);
#endif




