/* "NETGEN", a netlist-specification tool for VLSI
   Copyright (C) 1989, 1990   Massimo A. Sivilotti
   Author's address: mass@csvax.cs.caltech.edu;
                     Caltech 256-80, Pasadena CA 91125.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation (any version).

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file copying.  If not, write to
the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/* xillib.c -- definitions of Xilinx cells for ACTEL cells. */


/* define the following to make pads unique */
/* this is necessary for ntk2adl, but is annoying for PLACE */

#undef USE_UNIQUE_GLOBALS


#include "config.h"
#include <stdio.h>
#include "netgen.h"

static int xilinx_lib_present = 0;

int XilinxLibPresent(void)
{
  return xilinx_lib_present;
}

void XilinxLib(void)
{
  int OldDebug;

  OldDebug = Debug;
  Debug = 0;


  CellDef("OUTBUF");
     Primitive();
#ifdef USE_UNIQUE_GLOBALS
     UniqueGlobal("PAD");
#else
     Port("O");
#endif
     Port("I");
  EndCell();


  CellDef("INBUF");
     Primitive();
     Port("O");
#ifdef USE_UNIQUE_GLOBALS
     UniqueGlobal("PAD");
#else
     Port("I");
#endif
  EndCell();

  CellDef("CLKBUF");
     Primitive();
     Port("O");
#ifdef USE_UNIQUE_GLOBALS
     UniqueGlobal("PAD");
#else
     Port("I");
#endif
  EndCell();

  CellDef("CLOCK");
     Primitive();
     Port("O");
  EndCell();

  CellDef("TRIBUFF");
     Primitive();
#ifdef USE_UNIQUE_GLOBALS
     UniqueGlobal("PAD");
#else
     Port("O");
#endif
     Port("I");
     Port("!T");
  EndCell();


  CellDef("BIBUF");
     Primitive();
#ifdef USE_UNIQUE_GLOBALS
     UniqueGlobal("PAD");
#else
     Port("O");
#endif
     Port("I");
     Port("E");
     Port("IN");
  EndCell();

	CellDef("AND2");
		Primitive();
		Port("1");
		Port("2");
		Port("O");
	EndCell();

	CellDef("AND2A");
		Primitive();
		Port("!1");
		Port("2");
		Port("O");
	EndCell();

	CellDef("AND2B");
		Primitive();
		Port("!1");
		Port("!2");
		Port("O");
	EndCell();

	CellDef("NAND2");
		Primitive();
		Port("1");
		Port("2");
		Port("O");
	EndCell();

	CellDef("NAND2A");
		Primitive();
		Port("!1");
		Port("2");
		Port("O");
	EndCell();

	CellDef("NAND2B");
		Primitive();
		Port("!1");
		Port("!2");
		Port("O");
	EndCell();

	CellDef("OR2");
		Primitive();
		Port("1");
		Port("2");
		Port("O");
	EndCell();

	CellDef("OR2A");
		Primitive();
		Port("!1");
		Port("2");
		Port("O");
	EndCell();

	CellDef("OR2B");
		Primitive();
		Port("!1");
		Port("!2");
		Port("O");
	EndCell();

	CellDef("NOR2");
		Primitive();
		Port("1");
		Port("2");
		Port("O");
	EndCell();

	CellDef("NOR2A");
		Primitive();
		Port("!1");
		Port("2");
		Port("O");
	EndCell();

	CellDef("NOR2B");
		Primitive();
		Port("!1");
		Port("!2");
		Port("O");
	EndCell();

	CellDef("AND3");
		Primitive();
		Port("1");
		Port("2");
		Port("3");
		Port("O");
	EndCell();

	CellDef("AND3A");
		Primitive();
		Port("!1");
		Port("2");
		Port("3");
		Port("O");
	EndCell();

	CellDef("AND3B");
		Primitive();
		Port("!1");
		Port("!2");
		Port("3");
		Port("O");
	EndCell();

	CellDef("AND3C");
		Primitive();
		Port("!1");
		Port("!2");
		Port("!3");
		Port("O");
	EndCell();

	CellDef("NAND3");
		Primitive();
		Port("1");
		Port("2");
		Port("3");
		Port("O");
	EndCell();

	CellDef("NAND3A");
		Primitive();
		Port("!1");
		Port("2");
		Port("3");
		Port("O");
	EndCell();

	CellDef("NAND3B");
		Primitive();
		Port("!1");
		Port("!2");
		Port("3");
		Port("O");
	EndCell();

	CellDef("NAND3C");
		Primitive();
		Port("!1");
		Port("!2");
		Port("!3");
		Port("O");
	EndCell();

	CellDef("OR3");
		Primitive();
		Port("1");
		Port("2");
		Port("3");
		Port("O");
	EndCell();

	CellDef("OR3A");
		Primitive();
		Port("!1");
		Port("2");
		Port("3");
		Port("O");
	EndCell();

	CellDef("OR3B");
		Primitive();
		Port("!1");
		Port("!2");
		Port("3");
		Port("O");
	EndCell();

	CellDef("OR3C");
		Primitive();
		Port("!1");
		Port("!2");
		Port("!3");
		Port("O");
	EndCell();

	CellDef("NOR3");
		Primitive();
		Port("1");
		Port("2");
		Port("3");
		Port("O");
	EndCell();

	CellDef("NOR3A");
		Primitive();
		Port("!1");
		Port("2");
		Port("3");
		Port("O");
	EndCell();

	CellDef("NOR3B");
		Primitive();
		Port("!1");
		Port("!2");
		Port("3");
		Port("O");
	EndCell();

	CellDef("NOR3C");
		Primitive();
		Port("!1");
		Port("!2");
		Port("!3");
		Port("O");
	EndCell();

	CellDef("AND4");
		Primitive();
		Port("1");
		Port("2");
		Port("3");
		Port("4");
		Port("O");
	EndCell();

	CellDef("AND4A");
		Primitive();
		Port("!1");
		Port("2");
		Port("3");
		Port("4");
		Port("O");
	EndCell();

	CellDef("AND4B");
		Primitive();
		Port("!1");
		Port("!2");
		Port("3");
		Port("4");
		Port("O");
	EndCell();

	CellDef("AND4C");
		Primitive();
		Port("!1");
		Port("!2");
		Port("!3");
		Port("4");
		Port("O");
	EndCell();

	CellDef("AND4D");
		Primitive();
		Port("!1");
		Port("!2");
		Port("!3");
		Port("!4");
		Port("O");
	EndCell();

	CellDef("NAND4");
		Primitive();
		Port("1");
		Port("2");
		Port("3");
		Port("4");
		Port("O");
	EndCell();

	CellDef("NAND4A");
		Primitive();
		Port("!1");
		Port("2");
		Port("3");
		Port("4");
		Port("O");
	EndCell();

	CellDef("NAND4B");
		Primitive();
		Port("!1");
		Port("!2");
		Port("3");
		Port("4");
		Port("O");
	EndCell();

	CellDef("NAND4C");
		Primitive();
		Port("!1");
		Port("!2");
		Port("!3");
		Port("4");
		Port("O");
	EndCell();

	CellDef("NAND4D");
		Primitive();
		Port("!1");
		Port("!2");
		Port("!3");
		Port("!4");
		Port("O");
	EndCell();

	CellDef("OR4");
		Primitive();
		Port("1");
		Port("2");
		Port("3");
		Port("4");
		Port("O");
	EndCell();

	CellDef("OR4A");
		Primitive();
		Port("!1");
		Port("2");
		Port("3");
		Port("4");
		Port("O");
	EndCell();

	CellDef("OR4B");
		Primitive();
		Port("!1");
		Port("!2");
		Port("3");
		Port("4");
		Port("O");
	EndCell();

	CellDef("OR4C");
		Primitive();
		Port("!1");
		Port("!2");
		Port("!3");
		Port("4");
		Port("O");
	EndCell();

	CellDef("OR4D");
		Primitive();
		Port("!1");
		Port("!2");
		Port("!3");
		Port("!4");
		Port("O");
	EndCell();

	CellDef("NOR4");
		Primitive();
		Port("1");
		Port("2");
		Port("3");
		Port("4");
		Port("O");
	EndCell();

	CellDef("NOR4A");
		Primitive();
		Port("!1");
		Port("2");
		Port("3");
		Port("4");
		Port("O");
	EndCell();

	CellDef("NOR4B");
		Primitive();
		Port("!1");
		Port("!2");
		Port("3");
		Port("4");
		Port("O");
	EndCell();

	CellDef("NOR4C");
		Primitive();
		Port("!1");
		Port("!2");
		Port("!3");
		Port("4");
		Port("O");
	EndCell();

	CellDef("NOR4D");
		Primitive();
		Port("!1");
		Port("!2");
		Port("!3");
		Port("!4");
		Port("O");
	EndCell();


/*
INBUF
CLKBUF
OUTBUF
TRIBUF
BIBUF
*/





	CellDef("BUF");
		Primitive();
		Port("I");
		Port("O");
	EndCell();

	CellDef("BUFA");
		Primitive();
		Port("I");
		Port("O");
	EndCell();

	CellDef("INV");
		Primitive();
		Port("I");
		Port("O");
	EndCell();

	CellDef("INVA");
		Primitive();
		Port("I");
		Port("O");
	EndCell();

	CellDef("XOR");
		Primitive();
		Port("1");
		Port("2");
		Port("O");
	EndCell();

	CellDef("XNOR");
		Primitive();
		Port("1");
		Port("2");
		Port("O");
	EndCell();
/*
	CellDef("XO1");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("X01A");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("XA1");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("XA1A");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("AX1");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("AX1A");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("AX1B");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("AO1");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("AO1A");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("AO1B");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("AO1C");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("AOI1A");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("AOI1B");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("MAJ3");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("AO2");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("4");
		Port("Y");
	EndCell();

	CellDef("AO2A");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("4");
		Port("Y");
	EndCell();

	CellDef("AOI2A");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("4");
		Port("Y");
	EndCell();

	CellDef("AOI2B");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("4");
		Port("Y");
	EndCell();

	CellDef("OA1");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("OA1A");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("OA1B");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("OA1C");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("Y");
	EndCell();

	CellDef("OA3");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("D");
		Port("Y");
	EndCell();

	CellDef("OA3A");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("D");
		Port("Y");
	EndCell();

	CellDef("OA3B");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("D");
		Port("Y");
	EndCell();

	CellDef("OA2");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("D");
		Port("Y");
	EndCell();

	CellDef("OA2A");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("D");
		Port("O");
	EndCell();

	CellDef("MX2");
		Primitive();
		Port("A");
		Port("B");
		Port("S");
		Port("O");
	EndCell();

	CellDef("MX2A");
		Primitive();
		Port("A");
		Port("B");
		Port("S");
		Port("O");
	EndCell();

	CellDef("MX2B");
		Primitive();
		Port("A");
		Port("B");
		Port("S");
		Port("O");
	EndCell();

	CellDef("MX2C");
		Primitive();
		Port("A");
		Port("B");
		Port("S");
		Port("O");
	EndCell();

	CellDef("MX4");
		Primitive();
		Port("D0");
		Port("D1");
		Port("D2");
		Port("D3");
		Port("S1");
		Port("S0");
		Port("O");
	EndCell();

	CellDef("MXT");
		Primitive();
		Port("A");
		Port("B");
		Port("C");
		Port("D");
		Port("S0A");
		Port("S0B");
		Port("S1");
		Port("O");
	EndCell();

	CellDef("HA1");
		Primitive();
		Port("A");
		Port("B");
		Port("CO");
		Port("S");
	EndCell();

	CellDef("HA1A");
		Primitive();
		Port("A");
		Port("B");
		Port("CO");
		Port("S");
	EndCell();

	CellDef("HA1B");
		Primitive();
		Port("A");
		Port("B");
		Port("CO");
		Port("S");
	EndCell();

	CellDef("HA1C");
		Primitive();
		Port("A");
		Port("B");
		Port("CO");
		Port("S");
	EndCell();

	CellDef("FA1A");
		Primitive();
		Port("A");
		Port("B");
		Port("CI");
		Port("CO");
		Port("S");
	EndCell();

	CellDef("FA1B");
		Primitive();
		Port("A");
		Port("B");
		Port("CI");
		Port("CO");
		Port("S");
	EndCell();

	CellDef("FA2A");
		Primitive();
		Port("A0");
		Port("A1");
		Port("B");
		Port("CI");
		Port("CO");
		Port("S");
	EndCell();

	CellDef("DL1");
		Primitive();
		Port("D");
		Port("G");
		Port("Q");
	EndCell();

	CellDef("DL1A");
		Primitive();
		Port("D");
		Port("G");
		Port("QN");
	EndCell();

	CellDef("DL1B");
		Primitive();
		Port("D");
		Port("G");
		Port("Q");
	EndCell();

	CellDef("DL1C");
		Primitive();
		Port("D");
		Port("G");
		Port("QN");
	EndCell();

	CellDef("DLC");
		Primitive();
		Port("D");
		Port("G");
		Port("Q");
		Port("CLR");
	EndCell();

	CellDef("DLCA");
		Primitive();
		Port("D");
		Port("G");
		Port("Q");
		Port("CLR");
	EndCell();

	CellDef("DLE");
		Primitive();
		Port("D");
		Port("G");
		Port("Q");
		Port("E");
	EndCell();

	CellDef("DLEA");
		Primitive();
		Port("D");
		Port("G");
		Port("Q");
		Port("E");
	EndCell();

	CellDef("DLEB");
		Primitive();
		Port("D");
		Port("G");
		Port("Q");
		Port("E");
	EndCell();

	CellDef("DLEC");
		Primitive();
		Port("D");
		Port("G");
		Port("Q");
		Port("E");
	EndCell();

	CellDef("DLM");
		Primitive();
		Port("A");
		Port("B");
		Port("S");
		Port("G");
		Port("Q");
	EndCell();

	CellDef("DLMA");
		Primitive();
		Port("A");
		Port("B");
		Port("S");
		Port("G");
		Port("Q");
	EndCell();

	CellDef("JKF");
		Primitive();
		Port("J");
		Port("K");
		Port("CLK");
		Port("Q");
	EndCell();

	CellDef("JKFPC");
		Primitive();
		Port("J");
		Port("K");
		Port("CLK");
		Port("Q");
		Port("PRE");
		Port("CLR");
	EndCell();

	CellDef("TFC");
		Primitive();
		Port("T");
		Port("CLK");
		Port("Q");
		Port("CLR");
	EndCell();

	CellDef("DFM");
		Primitive();
		Port("A");
		Port("B");
		Port("S");
		Port("CLK");
		Port("Q");
	EndCell();

	CellDef("DFMA");
		Primitive();
		Port("A");
		Port("B");
		Port("S");
		Port("CLK");
		Port("Q");
	EndCell();

	CellDef("DFMB");
		Primitive();
		Port("A");
		Port("B");
		Port("S");
		Port("CLK");
		Port("Q");
		Port("CLR");
	EndCell();
*/

	CellDef("DF1");
		Primitive();
		Port("D");
		Port("C");
		Port("Q");
	EndCell();

	CellDef("DF1A");
		Primitive();
		Port("D");
		Port("C");
		Port("!Q");
	EndCell();

	CellDef("DF1B");
		Primitive();
		Port("D");
		Port("!C");
		Port("Q");
	EndCell();

	CellDef("DF1C");
		Primitive();
		Port("D");
		Port("!C");
		Port("!Q");
	EndCell();

	CellDef("DFC1");
		Primitive();
		Port("D");
		Port("C");
		Port("Q");
		Port("RD");
	EndCell();

	CellDef("DFC1A");
		Primitive();
		Port("D");
		Port("!C");
		Port("Q");
		Port("RD");
	EndCell();

	CellDef("DFC1B");
		Primitive();
		Port("D");
		Port("C");
		Port("Q");
		Port("!RD");
	EndCell();

	CellDef("DFC1C");
		Primitive();
		Port("D");
		Port("C");
		Port("!Q");
		Port("RD");
	EndCell();

	CellDef("DFC1D");
		Primitive();
		Port("D");
		Port("!C");
		Port("Q");
		Port("!RD");
	EndCell();

	CellDef("DFC1E");
		Primitive();
		Port("D");
		Port("C");
		Port("!Q");
		Port("!RD");
	EndCell();

	CellDef("DFC1F");
		Primitive();
		Port("D");
		Port("!C");
		Port("!Q");
		Port("RD");
	EndCell();

	CellDef("DFC1G");
		Primitive();
		Port("D");
		Port("!C");
		Port("!Q");
		Port("!RD");
	EndCell();
/*
	CellDef("DFP1");
		Primitive();
		Port("D");
		Port("CLK");
		Port("Q");
		Port("PRE");
	EndCell();

	CellDef("DFP1A");
		Primitive();
		Port("D");
		Port("CLK");
		Port("Q");
		Port("PRE");
	EndCell();

	CellDef("DFP1B");
		Primitive();
		Port("D");
		Port("CLK");
		Port("Q");
		Port("PRE");
	EndCell();

	CellDef("DFP1C");
		Primitive();
		Port("D");
		Port("CLK");
		Port("QN");
		Port("PRE");
	EndCell();

	CellDef("DFP1D");
		Primitive();
		Port("D");
		Port("CLK");
		Port("Q");
		Port("PRE");
	EndCell();

	CellDef("DFP1E");
		Primitive();
		Port("D");
		Port("CLK");
		Port("QN");
		Port("PRE");
	EndCell();

	CellDef("DFP1F");
		Primitive();
		Port("D");
		Port("CLK");
		Port("QN");
		Port("PRE");
	EndCell();

	CellDef("DFP1G");
		Primitive();
		Port("D");
		Port("CLK");
		Port("QN");
		Port("PRE");
	EndCell();

	CellDef("DFPC");
		Primitive();
		Port("D");
		Port("CLK");
		Port("Q");
		Port("CLR");
		Port("PRE");
	EndCell();

	CellDef("DFPCA");
		Primitive();
		Port("D");
		Port("CLK");
		Port("Q");
		Port("CLR");
		Port("PRE");
	EndCell();
*/

	CellDef("DFE");
		Primitive();
		Port("D");
		Port("C");
		Port("Q");
		Port("CE");
	EndCell();

	CellDef("DFEA");
		Primitive();
		Port("D");
		Port("!C");
		Port("Q");
		Port("CE");
	EndCell();

/*
	CellDef("DFEB");
		Primitive();
		Port("D");
		Port("C");
		Port("Q");
		Port("CE");
		Port("!RD");
		Port("PRE");
	EndCell();

	CellDef("DFEC");
		Primitive();
		Port("D");
		Port("CLK");
		Port("Q");
		Port("E");
		Port("CLR");
		Port("PRE");
	EndCell();

	CellDef("DFED");
		Primitive();
		Port("D");
		Port("C");
		Port("Q");
		Port("CE");
		Port("PRE");
	EndCell();
*/


  Debug = OldDebug;
  xilinx_lib_present = 1;
}
